/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Win32Thread Class & Sample Application.
 *
 * The Initial Developer of the Original Code is
 * Ehsan Akhgari <me@ehsanakhgari.org>.
 * Portions created by the Initial Developer are Copyright (C) 2003
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

// ThreadTestDlg.cpp : implementation file
//

#include "stdafx.h"
#include "ThreadTest.h"
#include "ThreadTestDlg.h"

#include "Win32Thread.h"

#include "TestProcThread.h"
#include "TestClassThread.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CThreadTestDlg dialog

CThreadTestDlg::CThreadTestDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CThreadTestDlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(CThreadTestDlg)
	m_strProcStatus = _T("");
	m_strClassStatus = _T("");
	//}}AFX_DATA_INIT
	// Note that LoadIcon does not require a subsequent DestroyIcon in Win32
	m_hIcon = AfxGetApp()->LoadIcon(IDR_MAINFRAME);
}

void CThreadTestDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CThreadTestDlg)
	DDX_Text(pDX, IDC_PROC_STATUS, m_strProcStatus);
	DDX_Text(pDX, IDC_CLASS_STATUS, m_strClassStatus);
	//}}AFX_DATA_MAP
}

BEGIN_MESSAGE_MAP(CThreadTestDlg, CDialog)
	//{{AFX_MSG_MAP(CThreadTestDlg)
	ON_WM_PAINT()
	ON_WM_QUERYDRAGICON()
	ON_BN_CLICKED(IDC_PROC_THREAD_START, OnProcThreadStart)
	ON_BN_CLICKED(IDC_CLASS_THREAD_START, OnClassThreadStart)
	//}}AFX_MSG_MAP
	ON_REGISTERED_MESSAGE(uwmProcUpdate, OnProcThreadProgress)
	ON_REGISTERED_MESSAGE(CThreadClass::s_uwmClassUpdate, OnClassThreadProgress)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CThreadTestDlg message handlers

BOOL CThreadTestDlg::OnInitDialog()
{
	CDialog::OnInitDialog();

	// Set the icon for this dialog.  The framework does this automatically
	//  when the application's main window is not a dialog
	SetIcon(m_hIcon, TRUE);			// Set big icon
	SetIcon(m_hIcon, FALSE);		// Set small icon
	
	// TODO: Add extra initialization here
	
	return TRUE;  // return TRUE  unless you set the focus to a control
}

// If you add a minimize button to your dialog, you will need the code below
//  to draw the icon.  For MFC applications using the document/view model,
//  this is automatically done for you by the framework.

void CThreadTestDlg::OnPaint() 
{
	if (IsIconic())
	{
		CPaintDC dc(this); // device context for painting

		SendMessage(WM_ICONERASEBKGND, (WPARAM) dc.GetSafeHdc(), 0);

		// Center icon in client rectangle
		int cxIcon = GetSystemMetrics(SM_CXICON);
		int cyIcon = GetSystemMetrics(SM_CYICON);
		CRect rect;
		GetClientRect(&rect);
		int x = (rect.Width() - cxIcon + 1) / 2;
		int y = (rect.Height() - cyIcon + 1) / 2;

		// Draw the icon
		dc.DrawIcon(x, y, m_hIcon);
	}
	else
	{
		CDialog::OnPaint();
	}
}

// The system calls this to obtain the cursor to display while the user drags
//  the minimized window.
HCURSOR CThreadTestDlg::OnQueryDragIcon()
{
	return (HCURSOR) m_hIcon;
}

void CThreadTestDlg::OnProcThreadStart() 
{
	// prepare thread parameters
	ThreadProcParam * pParams = new ThreadProcParam;
	pParams->hwnd = GetSafeHwnd();
	pParams->strThreadName = _T("The proc thread");

	// start the thread
	CWin32Thread * pThread = CWin32ProcThreadCreator::CreateThread( &::ThreadProc, pParams );

	if (pThread) // if thread started successfully
	{
		m_strProcStatus = _T("started successfully");
	}
	else
	{
		m_strProcStatus = _T("failed to start");
	}
	UpdateData( FALSE );
}

LRESULT CThreadTestDlg::OnProcThreadProgress(WPARAM, LPARAM secs)
{
	CString str;
	if (secs)
	{
		str.Format( _T("%d seconds to die!"), secs );
	}
	else
	{
		str = _T("finished");
	}

	m_strProcStatus = str;
	UpdateData( FALSE );

	return 0;
}

void CThreadTestDlg::OnClassThreadStart() 
{
	// start the thread as paused
	CThreadClass * pThread = CWin32ThreadCreator< CThreadClass >::CreateThread(
		THREAD_PRIORITY_NORMAL, 0, CREATE_SUSPENDED );

	if (pThread) // if thread started successfully
	{
		CWin32ThreadInfo m_myThread;
		m_myThread = *pThread;

		// adjust its parameters
		pThread->SetHWnd( GetSafeHwnd() );
		pThread->SetThreadName( _T("The class thread") );

		// resume the thread (so that it starts its job)
		if (pThread->ResumeThread())
		{
			m_strClassStatus = _T("started successfully");
		}
		else
		{
			m_strClassStatus = _T("failed to start");
		}
	}
	else
	{
		m_strClassStatus = _T("failed to start");
	}
	UpdateData( FALSE );
}

LRESULT CThreadTestDlg::OnClassThreadProgress(WPARAM, LPARAM secs)
{
	CString str;
	if (secs)
	{
		str.Format( _T("%d seconds to die!"), secs );
	}
	else
	{
		str = _T("finished");
	}

	m_strClassStatus = str;
	UpdateData( FALSE );

	return 0;
}
